from pprint import pprint

from bs4 import BeautifulSoup
from requests import get

from util.config import get_config
from util.utils import format_output, cast, run_thread, singlify


def affiliation_authors(affiliation_id, output_format='dict'):
    affiliation_id = singlify(affiliation_id)
    domain = get_config()['domain']
    url = f'{domain}/affiliations/authors/{affiliation_id}'
    html = BeautifulSoup(get(url).content, 'html.parser')
    n_pages = cast(html.select('.pagination-text')[0].text.split(' ')[3])
    result = parse(html)
    thread_result = run_thread(worker, list(range(2, n_pages + 1)), affiliation_id=affiliation_id)

    result.extend(thread_result)

    return format_output(result, output_format)


def worker(page, result, **kwargs):
    affiliation_id = kwargs['affiliation_id']
    domain = get_config()['domain']
    url = f'{domain}/affiliations/authors/{affiliation_id}?page={page}'
    html = BeautifulSoup(get(url).content, 'html.parser')
    data = parse(html)

    result.extend(data)


def parse(html):
    rows = html.select('.au-item')
    result = []

    for row in rows:
        profile_picture = row.select('img')[0]['src']
        profile = row.select('.profile-name > a')[0]
        profile_url = profile['href']
        profile_name = profile.text
        profile_id = row.select('.profile-id')[0].text.split(':')[1].strip()

        department = row.select('.profile-dept > a')[0]
        department_url = department['href']
        department_name = department.text.strip()

        h_index_row = row.select('.profile-hindex > .profile-id')
        h_index_numbers = [cast(h_index_row[i].text.split(':')[1].strip()) for i in (0, 1)]
        h_index = dict(zip(('scopus', 'scholar'), h_index_numbers))

        score_names = 'sinta_3_years', 'sinta', 'affil_3_years', 'affil'
        score_numbers = [cast(row.select('.stat-num')[i].text.replace('.', '')) for i in range(4)]
        scores = dict(zip(score_names, score_numbers))

        result.append({
            'profile_picture': profile_picture,
            'id': profile_id,
            'url': profile_url,
            'name': profile_name,
            'department': {
                'url': department_url,
                'name': department_name
            },
            'h_index': h_index,
            'scores': scores
        })

    return result


if __name__ == '__main__':
    data = affiliation_authors(404)

    pprint(data)



from bs4 import BeautifulSoup
from requests import get

import sinta
from util.config import get_config
from util.utils import format_output, listify, run_thread, compact_list, cast


def department(department_ids, affiliation_id, output_format='dict'):
    affiliation_code = sinta.affiliation(affiliation_id)['code']
    result = run_thread(worker, listify(department_ids), affiliation_id=affiliation_id,
                        affiliation_code=affiliation_code)
    result = compact_list(result)

    return format_output(result, output_format)


def worker(department_id, worker_result, **kwargs):
    affiliation_id = kwargs['affiliation_id']
    affiliation_code = kwargs['affiliation_code']
    domain = get_config()['domain']
    url = f'{domain}/departments/profile/{affiliation_id}/{affiliation_code}/{department_id}'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')

    name = soup.select('.univ-name > h3')[0].text.strip()
    location = soup.select('.affil-loc')[0].text.strip()
    affiliation_soup = soup.select('.meta-profile > a')[0]
    affiliation = {
        'id': cast(affiliation_soup['href'].split('/')[-1]),
        'name': affiliation_soup.text.strip(),
        'url': affiliation_soup['href']
    }

    result_data = {
        'id': department_id,
        'name': name,
        'location': location,
        'url': url,
        'affiliation': affiliation
    }

    worker_result.append(result_data)



from bs4 import BeautifulSoup
from requests import get

from util.config import get_config
from util.utils import format_output, cast, listify, run_thread, compact_list


def affiliation(affiliation_ids, output_format='dict'):
    result = run_thread(worker, listify(affiliation_ids))
    result = compact_list(result)

    return format_output(result, output_format)


def worker(affiliation_id, worker_result):
    domain = get_config()['domain']
    url = f'{domain}/affiliations/profile/{affiliation_id}'
    html = BeautifulSoup(get(url).content, 'html.parser')

    # profile section
    name = html.select('.univ-name > h3')[0].text.strip()
    abbrv_name = html.select('.affil-abbrev')[0].text.strip()
    location = html.select('.affil-loc')[0].text.strip()
    code = html.select('.affil-code')[0].text.split(':')[-1].strip()

    # stats section
    stat_profile = html.select('.affil-profile-card .stat-num')
    stat_names = 'authors', 'departments', 'journals'
    stat_int = [cast(stat_profile[i].text.replace('.', '')) for i in range(3)]
    stats = dict(zip(stat_names, stat_int))

    # sinta score section
    score_soup = html.select('.stat-profile .pr-num')
    score_names = 'overall', '3_years', 'productivity', 'productivity_3_years'
    score_numbers = [cast(score_soup[i].text.replace('.', '')) for i in range(4)]
    sinta_scores = dict(zip(score_names, score_numbers))

    # indexer section
    index_stats = {}
    index_rows = html.select('.stat-table > tbody > tr')
    index_aspects = 'articles', 'citations', 'cited_documents', 'citation_per_researcher'
    indexers = 'scopus', 'scholar', 'wos', 'garuda'

    last_update = html.select('small')[-1].text.split(' :')[1]

    for i, row in enumerate(index_rows):
        numbers = [cast(row.select('td')[i].text.replace('.', '').replace(',', '.')) for i in range(1, 5)]
        index_stats[index_aspects[i]] = dict(zip(indexers, numbers))

    result_data = {
                      'id': affiliation_id,
                      'code': code,
                      'url': url,
                      'name': name,
                      'abbreviation': abbrv_name,
                      'location': location,
                      'sinta_score': sinta_scores,
                      'last_update': last_update
                  } | stats | index_stats

    worker_result.append(result_data)



from bs4 import BeautifulSoup
from requests import get

from util.config import get_config
from util.utils import format_output, cast, listify, run_thread, compact_list


def author(author_ids, output_format='dict'):
    author_ids = listify(author_ids)
    result = run_thread(worker, author_ids)
    result = compact_list(result)

    return format_output(result, output_format)


def worker(author_id, worker_result):
    domain = get_config()['domain']
    url = f'{domain}/authors/profile/{author_id}'
    html = get(url)
    soup = BeautifulSoup(html.content, 'html.parser')

    name = soup.select('h3 > a')[0].text.strip()
    profile = soup.select('.meta-profile a')
    affil_name = profile[0].text.strip()
    affil_url = profile[0]['href'].strip()
    affil_id = cast(affil_url.split('/')[-1])
    dept = profile[1].text.strip()
    subjects = [subject.text.strip() for subject in soup.select('.subject-list a')]

    scores_soup = soup.select('.stat-profile .pr-num')
    score_names = 'overall', '3_years', 'affiliation', 'affiliation_3_years'
    scores_int = [cast(scores_soup[i].text) for i in range(4)]
    scores = dict(zip(score_names, scores_int))

    stat_names = 'articles', 'citations', 'cited_docs', 'h-index', 'i10-index', 'g-index'
    indexers = 'scopus', 'scholar', 'wos'
    stat_soup = soup.select('.stat-table > tbody > tr')
    stats = {}

    for i, stat_name in enumerate(stat_names):
        s = [cast(stat_soup[i].select('td')[j].text.strip()) for j in range(1, 4)]
        stats[stat_name] = dict(zip(indexers, s))

    result_data = {
                      'id': author_id,
                      'name': name,
                      'url': url,
                      'affiliation': {
                          'id': affil_id,
                          'name': affil_name,
                          'url': affil_url
                      },
                      'department': dept,
                      'subjects': subjects,
                      'score': scores
                  } | stats

    worker_result.append(result_data)



import json

from dict2xml import dict2xml
from flatdict import FlatterDict
from string_utils.validation import is_integer, is_decimal


def format_output(data, output_format: str):
    if output_format == 'dict-flat':
        return dict(FlatterDict(data, delimiter='.'))
    elif output_format == 'json':
        return json.dumps(data)
    elif output_format == 'json-pretty':
        return json.dumps(data, indent=4)
    elif output_format == 'xml':
        return dict2xml(data, wrap='author')
    elif output_format == 'xml-flat':
        flat = format_output(data, output_format='dict-flat')
        return format_output(flat, output_format='xml')
    else:
        return data


def cast(string: str):
    string = string.strip()

    if is_integer(string):
        return int(string)
    elif is_decimal(string):
        return float(string)
    elif string == '-':
        return None
    else:
        return string


def listify(val) -> list:
    return [val] if type(val) not in (list, tuple) else val


def compact_list(_list: list):
    return _list[0] if len(_list) == 1 else _list


def singlify(val):
    return val[0] if type(val) in (list, tuple) else val


def run_thread(worker, iterable, **kwargs):
    from concurrent.futures import ThreadPoolExecutor

    result = []

    with ThreadPoolExecutor() as executor:
        for i in iterable:
            executor.submit(worker, i, result, **kwargs)

    return result



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='sinta-scraper',
    version='2.2.1',
    author='Randy Cahya Wihandika',
    author_email='rendicahya@gmail.com',
    description='Retrieves information from Sinta (Science and Technology Index) via scraping.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/rendicahya/sinta-scraper',
    packages=setuptools.find_packages(),
    install_requires=['beautifulsoup4', 'requests', 'dict2xml', 'python-string-utils', 'flatdict'],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
